{{/*
Common labels
*/}}
{{- define "dynamodb.labels" -}}
{{ include "dynamodb-utils.labels" . }}
{{- end -}}

{{/*
Selector labels
*/}}
{{- define "dynamodb.selectorLabels" -}}
{{ include "dynamodb-utils.selectorLabels" . }}
{{- end -}}

{{/*
    The table prefix for the Database
    We don't use the "standard" utils.externalNamingPrefix here because we want
    to allow apps to have a common prefix across multiple ~shared tables.
    Region is not required as DynamoDB tables are scoped within a region/account.

    Set Values.releaseNameOverride to use an alternate release name when refactoring
    instantiaion charts to maintain backwards compatibility with existing names.
*/}}
{{- define "dynamodb.tablePrefix" -}}
{{- .Values.global.env -}}-
{{- if .Values.releaseNameOverride -}}
{{- .Values.releaseNameOverride -}}
{{- else -}}
{{- .Release.Name -}}
{{- end -}}
{{- end -}}


{{/*
    - Generates the name of the dynamodb table
    - Checks if the length of the dyanmodb table
    is within range of the number of characters allowed
    - If not errors out
*/}}
{{- define "dynamodb.tableName" -}}
{{- $tableName:= printf "%s%s%s" (include "dynamodb.tablePrefix" .) ( .Values.tableNamePrefixSeperator ) (required "You must specify the 'tableName' to be created" .Values.tableName) -}}
{{- $lenTableName := $tableName | len | float64 -}}
{{- if ge $lenTableName .Values.maxNameLength -}}
{{- printf "\nThe generated tableName %s is greater than the allowed number of characters %f." $tableName .Values.maxNameLength | fail -}}
{{- else -}}
{{- printf "%s" $tableName -}}
{{- end -}}
{{- end -}}

{{/*
    Checks if the billingMode  mentioned in the values file is valid.
    It should be a value from the list: [PROVISIONED, PAY_PER_REQUEST]
    Error out if invalid value is specified
    For Local Deployment throw an error if the value for billingMode is
    PROVISIONED. LocalStack does not support it yet.
*/}}
{{- define "dynamodb.checkbillingModeValid" -}}
{{- $billingMode :=  required "You must specify the value for .Values.billingMode " .Values.billingMode -}}
{{- $allowedValues := list "PROVISIONED" "PAY_PER_REQUEST" -}}
{{- if not ( has $billingMode $allowedValues ) -}}
{{- printf "\n\nInvalid Value set for billingMode: %s. The Allowed Values are: %s for AWS and only \"PAY_PER_REQUEST\" for Local" $billingMode $allowedValues  | fail -}}
{{- end -}}
{{- if and (eq .Values.global.awsDeployment false) (eq $billingMode "PROVISIONED")  -}}
{{- printf "\n\n\"PROVISIONED\" billingMode is not supported for local deployments. Please set the value for .Values.billingMode as \"PAY_PER_REQUEST\"" | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Checks if the streamViewType mentioned in the values file is valid.
    It should be a value from the list: [ALL, INCLUDE and KEYS_ONLY]
    Error if invalid value is specified
*/}}
{{- define "dynamodb.checkStreamViewTypeValid" -}}
{{- $streamViewType :=  .Values.streamViewType  -}}
{{- $allowedValues := list "KEYS_ONLY" "NEW_IMAGE" "OLD_IMAGE" "NEW_AND_OLD_IMAGES" -}}
{{- if not ( has $streamViewType $allowedValues ) -}}
{{- printf "\n\nInvalid Value set for .Values.streamViewType: %s. The Allowed Values are: %s" $streamViewType $allowedValues | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Checks if the projection type mentioned in the values file is valid.
    It should be a value from the list: [ALL, INCLUDE and KEYS_ONLY]
    Error if invalid value is specified
*/}}
{{- define "dynamodb.checkProjectionTypeValid" -}}
{{- $projectionType := (required "You must specify the value for projectionType of GSI" .projectionType) -}}
{{- $allowedValues := list "ALL" "INCLUDE" "KEYS_ONLY" -}}
{{- if not ( has $projectionType $allowedValues ) -}}
{{- printf "\n\nInvalid Value set for projectionType of GSI: %s. The Allowed Values are: %s" $projectionType $allowedValues | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Check if NonKeyAttributes are needed
    This attribute is required only when the
    value of projectionType is "INCLUDE", should throw an error otherwise
*/}}
{{- define "dynamodb.checkIfNonKeyAttributesRequired" -}}
{{- if not (eq .projectionType "INCLUDE") -}}
{{- printf "\n\nValue for nonKeyAttributes should not be set when the value for projectionType is %s. This contract is accepted only when the value for projectionType is \"INCLUDE\"" .projectionType | fail -}}
{{- end -}}
{{- end -}}


{{/*
    Check if there is a value set for either of the "attributes" input contract
    Fail if all of the following: stringAttributes, NumberAttributes and binaryAttributes are empty
*/}}
{{- define "dynamodb.checkAttributesValid" -}}
{{- if not (or .Values.stringAttributes .Values.numberAttributes .Values.binaryAttributes ) -}}
{{- printf "\n\nEither one of the following attributes should be set: \"stringAttributes\", \"numberAttributes\", \"binaryAttributes\". They should not all be empty" | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Configure list of DynamoDB GSI dictionaries.
    Forms a list of dictionary in a format that can be consumed by terraform
    as a variable of type: list(map(any)).

*/}}
{{- define "dynamodb.globalSecondaryIndexes" -}}
{{- $billingMode := .Values.billingMode -}}
{{- $gsi :=  .Values.globalSecondaryIndex -}}
[   {{ range $gsi  }}
    {
        name = {{ required "You must specify the name of the GSI index" .name | quote }}
        hash_key = {{ required "You must specify the hashKey of the GSI index" .hashKey | quote }}
        {{ if .rangeKey }}
        range_key = {{ .rangeKey | quote }}
        {{ end }}

        {{ include "dynamodb.checkProjectionTypeValid" . }}
        projection_type = {{ .projectionType | quote }}

        {{ if .nonKeyAttributes }}
        {{ include "dynamodb.checkIfNonKeyAttributesRequired" . }}
        non_key_attributes = {{ .nonKeyAttributes }}
        {{ end }}

        read_capacity = {{ .readCapacity | default 150 }}
        write_capacity = {{ .writeCapacity | default 150 }}


    },{{ end }}

]
{{- end -}}

{{/*
    Creates a map of autoscaling attributes for each GSI
    where in each key corresponds to the name of GSI and value includes
    all the autoscaling attributes.

*/}}
{{- define "dynamodb.autoscalingIndexes" }}
{{- $gsi :=  .Values.globalSecondaryIndex -}}
{
    {{ range $gsi }}
    {{ .name }} = {
        read_utilization_target = {{ .readUtilizationTarget | default 70 }}
        write_utilization_target = {{ .writeUtilizationTarget | default 70 }}
        min_read_capacity = {{ .minReadCapacity | default 5 }}
        max_read_capacity = {{ .maxReadCapacity | default 4000 }}
        min_write_capacity = {{ .minWriteCapacity | default 5 }}
        max_write_capacity = {{ .maxWriteCapacity | default 4000 }}
    },{{ end }}
}
{{- end -}}

{{- define "dynamodb.outputTableName" -}}
{{- if .Values.existingInfraOverrides.enable -}}
tableName: {{ .Values.existingInfraOverrides.tableName }}
{{- else -}}
tableName: "{{ include "dynamodb.tablePrefix" . }}{{ .Values.tableNamePrefixSeperator }}{{ required "You must specify the 'tableName' to be created" .Values.tableName }}"
{{- end -}}
{{- end -}}

{{/*
    Checks if the tableClass mentioned in the values file is valid.
    It should be a value from the list: [STANDARD and STANDARD_INFREQUENT_ACCESS]
    Error if invalid value is specified
*/}}
{{- define "dynamodb.checkTableClassValid" -}}
{{- $tableClass :=  .Values.tableClass  -}}
{{- $allowedValues := list "STANDARD" "STANDARD_INFREQUENT_ACCESS" -}}
{{- if not ( has $tableClass $allowedValues ) -}}
{{- printf "\n\nInvalid Value set for .Values.tableClass: %s. The Allowed Values are: %s" $tableClass $allowedValues | fail -}}
{{- end -}}
{{- end -}}